"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Commands related to controlling the camera's operation
"""

import google.protobuf.descriptor

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor
