# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Logger and other utility functionality."""

from .util import (
    Singleton,
    SnapshotQueue,
    add_cli_args_and_parse,
    ainput,
    cmd,
    deeply_update_dict,
    get_current_dst_aware_time,
    is_dataclass_instance,
    map_keys,
    pretty_print,
    scrub,
    to_dict,
)
